// $Id: tsplib.h,v 1.6 2008/11/25 22:58:31 xulin730 Exp $

#ifndef TSPLIB_H
#define TSPLIB_H 1

#include <stdio.h>
#include <fstream>
#include<set>
#include<map>
#include <vector>
#include "stopwatch.h"
#include "BinSolver.h"
#include <string.h>
#include <sstream>
#include <algorithm>


#define BUFFER_SIZE 4096
#define DEFAULT     -512

#define TSPLIB_OK                          0
#define TSPLIB_ERROR_NOT_SUPPORTED_TAG     1
#define TSPLIB_ERROR_NOT_SUPPORTED_VALUE   2
#define TSPLIB_ERROR_NO_VALID_PARAMETERS   3
#define TSPLIB_ERROR_FILEIO                4

#define TSPLIB_INVALID                   (-1)

#define TSPLIB_TYPE_TSP                    0
#define TSPLIB_TYPE_ATSP                   1
#define TSPLIB_TYPE_SOP                    2
#define TSPLIB_TYPE_HCP                    3
#define TSPLIB_TYPE_CVRP                   4
#define TSPLIB_TYPE_TOUR                   5

#define TSPLIB_EXPICIT                     0
#define TSPLIB_EUC_2D                      1
#define TSPLIB_EUC_3D                      2

#define TSPLIB_NO_COORDS                   0
#define TSPLIB_TWOD_COORDS                 1
#define TSPLIB_THREED_COORDS               2
#define NRUNS                              10
#define BC_TIME                            2
#define R_WALK                             100000
 using namespace std;

#define tsplib_max_num_tours 16
#define tsplib_type_cnt 6
static const char *tsplib_type_str[ tsplib_type_cnt ] =
    { "TSP", "ATSP", "SOP", "HCP", "CVRP", "TOUR" };
#define tsplib_edge_weight_type_cnt 3
static const char *tsplib_edge_weight_type_str[ tsplib_edge_weight_type_cnt ] =
    { "EXPLICIT", "EUC_2D", "EUC_3D" };
#define tsplib_node_coord_type_cnt 3
static const char *tsplib_node_coord_type_str[ tsplib_node_coord_type_cnt ] =
    { "NO_COORDS", "TWOD_COORDS", "THREED_COORDS" };

/** Memory structure for a TSPLIB file's specification
 */
 
int str_to_i(string newstr);
 
double str_to_f(string newstr);

int pair_comp (pair <int, int> p1, pair <int, int> p2);

void BuildLS(char* tourfile, char* finalfile, char* strseed);
void BuildBC(char* lptour, char* strseed);

struct specification
{
    string name;
    int type;
    string comment;
    int dimension;
    int capacity;
    int edge_weight_type;
    string edge_weight_format;
    string edge_data_format;
    int node_coord_type;
    string display_data_type;
};

/** Memory structure for a TSPLIB file's node coord section
 */
struct mynode_coord_section
{
    int count;
    int *index;
    double *cx, *cy, *cz;
};

struct sts
{
      int step;
      double time;
      double sol;
};

struct edge {
            double length;
            int a;
            int b;
            
            edge(int a_, int b_, double length_) {
                     a = a_;
                     b = b_;
                     length = length_;
                     };
                     
            bool operator >(const edge& right) const {
                  return length > right.length;
            };
     };
     

/** Memory structure for a TSPLIB file
 */
class TSPinstance
{
      public:
             TSPinstance (char *tspfilename);
             ~TSPinstance ();
             //TSPinstance (const TSPinstance& orig);
     	     //TSPinstance& operator=(const TSPinstance& orig);
             void outputSpec ();
             void comp_std ();
             void comp_mst();
             void display_mst();
             void comp_acf();
             void cluster_distance();
             void ls_prob(const char* mypath);
             void bc_prob(const char* mypath);
             char* filename;
             
             
             void writeFeature(const char *name, double val);
             void writeFeaturesToFile(const char *name);  // write comma separated features to passed file
             void writeFeaturesToFile(FILE* f);
             void writeFeatNamesToFile(const char *name); // write comma separated feature names to passed file
             void writeFeatNamesToFile(FILE* f);
             
      private:
             struct specification *spec;
             struct mynode_coord_section *node_coord_section;
             void get_key_value_pair ( char *inputline, char *key, char *value );
             void get_key_value_pair (string inputline, string& key, string& value );
             int parse_data_NODE_COORD_SECTION (ifstream &input);
             int parse_key_value_pair (char *key, char *value );
             double comp_dist(int i, int j);
             void comp_ass(int num, vector<double> myv, double normterm, double& avg, double& std, double& skew);
             void comp_ass(int num, double* myarray, double normterm, double& avg, double& std, double& skew);
             vector<double> comp_q(vector<double> myv);
             double cost_std;
             double cost_avg;
             double cost_skew;
             int *mst_visited;
             double *mst_d;
             int *mst_parent;
             double mst_cost;
             int** mst_tree;
             set<int> *mysets;
             vector<pair<double, double> > mst_cd;
             double max_cd, min_cd;
             double cd_std, cd_avg, cd_skew;
             Stopwatch gSW;
             char* strseed;
             long seed;
             char* tourfile;
             char* lptour;
             char* finalfile;
             void ls_outFileProcess(char* outFile);
             void ls_tourFileProcess(char* tourFile);
             void bc_tourFileProcess(char* lptour);
             void bc_outFileProcess(char* outFile);
             void ls_finalFileProcess(char* finalFile);
             vector<double> lpedgefreq;
    //         double lp_min, lp_max, lp_q25, lp_q50, lp_q75, lp1_p, lpn1_p;
             BinSolver* LSsolver;
             BinSolver* BCsolver;
             void BuildLS(char* tourfile, char* finalfile, char* strseed);
             void BuildBC(char* lptour, char* strseed);
            // map<string, int> nameToIndex; 
             char *featureNames[100];  // list of all feature names
             int indexCount;	      // count of how many features we currently have (built up using writeFeature() )
             double featureVals[100];  // the array of actual feature  values
             
            
             vector<double> tch_cost;
             double tch_avg;
             double tch_std;
             double tch_skew;
                          
             vector<double> lp_avg_imp_per_step;
             vector<double> lp_steps2lm;
             vector<double> lp_bestsol;
             
              vector<double> max_dist;
              
              
          
};

/** Reads and parses a TSPLIB file from a file handle and write data into a tsplibfile structure.
 * @param input File handle to read from.
 * @param tsplibfile Structure to write data into.
 * @return TSPLIB_OK if succeeded, else error code.
 */
// int read_tsplibfile ( FILE * input, struct tsplibfile **tsplibfile );

/** Writes a tsplibfile structure into a file (handle).
 * @param output File handle to write to.
 * @param tsplibfile Structure to read data from.
 */
// void write_tsplibfile ( FILE * output, struct tsplibfile *tsplibfile );

/** Frees the memory of a tsplibfile structure.
 */
// void free_struct_tsplibfile ( struct tsplibfile *tsplibfile );

#endif /* TSPLIB_H */
